==> kom%2B%2B-0.3.tar.gz: kom++-0.3/libkom++/session.cc <==
// Highlevel handling of a session to a LysKOM server.
//
// $Id: session.cc,v 1.19 1994/08/10 07:20:14 inge Exp $
// Copyright (C) 1994  Per Cederqvist and Inge Wallin
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

#include <stddef.h>
#include <iostream.h>
#include <assert.h>

#include "session.h"

#include "login_q.h"
#include "get_time_q.h"
#include "change_what_i_am_doing_q.h"
#include "lookup_name_q.h"
#include "re_lookup_person_q.h"
#include "get_marks_q.h"
#include "who_am_i_q.h"
#include "create_person_q.h"
#include "add_member_q.h"
#include "send_message_q.h"

// ================================================================ 
//                    Constructors and Destructors                  


Session::Session(Connection *connection)
    : conn(connection), all_confs(NULL), all_persons(NULL), 
      all_maps(NULL), all_texts(NULL), logged_in_person(0),
      session_number(0)
{
    assert(connection != NULL);

    m_async_dispatcher = new Async_dispatcher(connection);
}
 
// Session::Session(const Session &s)
//     : conn(s.conn), all_confs(s.all_confs), all_persons(s.all_persons), 
//       all_maps(s.all_maps), all_texts(s.all_texts),
//       logged_in_person(s.logged_in_person), session_number(s.session_number)
// {
// }


Session::~Session()
{
    if (all_confs != NULL)
	delete all_confs;
    if (all_persons != NULL)
	delete all_persons;
    if (all_maps != NULL)
	delete all_maps;
    if (all_texts != NULL)
	delete all_texts;
    if (m_async_dispatcher != NULL)
	delete m_async_dispatcher;
}


// ================================================================ 
//

// const Session & 
// Session::operator = (const Session &s)
// {
//     conn    = s.conn;
//     all_confs   = s.all_confs;
//     all_persons = s.all_persons;
//     all_maps    = s.all_maps;
//     all_texts   = s.all_texts;
// 
//     logged_in_person = s.logged_in_person;
//     session_number   = s.session_number;
// 
//     return *this;
// }


// ostream &
// operator <<(ostream &os, const Session &s)
// {
//     cout << "A session object";
//     return os;
// }


//void
//Session::set_connection(Connection *connection)
//{
//    conn = connection;
//    //+++ Reset confs, persons, maps and texts here.
//}


// ================================================================


Conferences & 
Session::conferences()
{
    assert(conn != NULL);

    if (all_confs == NULL) {
	all_confs = new Conferences(conn);
    }

    return *all_confs;
}


Persons &
Session::persons()
{
    if (all_persons == NULL) {
	all_persons = new Persons(conferences());
    }

    return *all_persons;
}


Maps &
Session::maps()
{
    if (all_maps == NULL) {
	all_maps = new Maps(conferences());
    }

    return *all_maps;
}


Texts &  
Session::texts()
{
    assert(conn != NULL);

    if (all_texts == NULL) {
	all_texts = new Texts(conn);
    }

    return *all_texts;

}


// ----------------------------------------------------------------

Status
Session::create_person(const LyStr &name,
		       const LyStr &password,
		       Pers_no *result,
		       Bool *autologin)
{
    create_person_question q(conn, name, password);
    
    Status ret = q.receive();
    if (ret == st_ok)
    {
	if (result != NULL)
	    *result = q.result();
	// If you are not already logged in, an automatic login occurs.
	if (logged_in_person == 0)
	{
	    logged_in_person = q.result();
	    if (autologin != NULL)
		*autologin = TRUE;
	}
	else
	    if (autologin != NULL)
		*autologin = FALSE;
    }

    return ret;
}

Status
Session::login(Pers_no person, const LyStr &passwd, Bool invisible)
{
    login_question in(conn, person, passwd, invisible);

    // +++ Delete all caches!  Should this be a separate method?
    Status ret = in.receive();
    if (ret == st_ok)
	logged_in_person = person;
    else
	logged_in_person = 0;

    return ret;
}

Status
Session::add_member(Conf_no cno, 
		    Pers_no reader,
		    unsigned char priority, 
		    unsigned short placement)
{
    add_member_question q(conn, cno, reader, priority, placement);
    return q.receive();
}

Status
Session::send_message(Pers_no recipient, 
		      const LyStr &message)
{
    // FIXME: Loses error code.
    send_message_question q(conn, recipient, message);
    return q.receive();
}

struct tm
Session::get_time()
{
    get_time_question   timeq(conn);
    Status              stat = timeq.receive();

    if (stat == st_ok)
	return timeq.result();

    // +++ Error handling!!
    return timeq.result();
}


Status
Session::change_what_i_am_doing(const LyStr &str)
{
    change_what_i_am_doing_question  doing(conn, str);

    return doing.receive();
}


Array<Who_info>
Session::who_is_on()
{
    who_is_on_question   who_q(conn);
    Array<Who_info>      arr;
    
    if (who_q.receive() == st_ok) {
	arr = who_q.result();
	return arr;
    } else 
	abort();		// +++Error handling!
}


Array<Micro_conf>
Session::lookup_name(const LyStr &pattern)
{
    lookup_name_question   lookupq(conn, pattern);

    if (lookupq.receive() == st_ok) 
	return lookupq.result();
    else
	abort();		// +++Error handling!
}


Array<Pers_no>
Session::regexp_lookup_persons(const LyStr &pattern)
{
    re_lookup_person_question   lookupq(conn, pattern);

    if (lookupq.receive() == st_ok) 
	return lookupq.result();
    else
	abort();		// +++Error handling!
}


Array<Mark>
Session::get_marks()
{
    get_marks_question  markq(conn);

    if (markq.receive() == st_ok) 
	return markq.result();
    else
	abort();		// +++Error handling!
}

Pers_no
Session::my_login() const
{
    return logged_in_person;
}

Session_no
Session::my_session() const
{
    if (session_number == 0)
    {
	who_am_i_question wq(conn);
	if (wq.receive() == st_ok)
	    ((Session*)this)->session_number = wq.result();
	else
	    abort();		// FIXME: Error handling
    }

    return session_number;
}


Async_dispatcher &
Session::async_dispatcher()
{
    return *m_async_dispatcher;
}


Status
Session::handle_async(int this_many)
{
    return m_async_dispatcher->handle_async(this_many);
}

==> kom%2B%2B-0.4.tar.gz: kom++-0.4/libkom++/session.cc <==
// Highlevel handling of a session to a LysKOM server.
//
// $Id: session.cc,v 1.27 1995/12/03 00:05:25 ceder Exp $
// Copyright (C) 1994  Per Cederqvist and Inge Wallin
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#include <stddef.h>
#include <iostream.h>
#include <assert.h>

#include "session.h"

#include "login_q.h"
#include "get_time_q.h"
#include "change_what_i_am_doing_q.h"
#include "lookup_name_q.h"
#include "lookup_z_name_q.h"
#include "re_lookup_person_q.h"
#include "get_marks_q.h"
#include "who_am_i_q.h"
#include "create_person_q.h"
#include "add_member_q.h"
#include "send_message_q.h"

// ================================================================ 
//                    Constructors and Destructors                  


Session::Session(Connection *connection) 
    : conn(connection),
      logged_in_person(0),
      session_number(0),
      all_confs(NULL),
      all_persons(NULL),
      all_maps(NULL),
      all_texts(NULL)
{
    assert(connection != NULL);

    m_async_dispatcher = new Async_dispatcher(connection);
}
 
// Session::Session(const Session &s)
//     : conn(s.conn), all_confs(s.all_confs), all_persons(s.all_persons), 
//       all_maps(s.all_maps), all_texts(s.all_texts),
//       logged_in_person(s.logged_in_person), session_number(s.session_number)
// {
// }


Session::~Session()
{
    if (all_confs != NULL)
	delete all_confs;
    if (all_persons != NULL)
	delete all_persons;
    if (all_maps != NULL)
	delete all_maps;
    if (all_texts != NULL)
	delete all_texts;
    if (m_async_dispatcher != NULL)
	delete m_async_dispatcher;
}


// ================================================================ 
//

// const Session & 
// Session::operator = (const Session &s)
// {
//     conn    = s.conn;
//     all_confs   = s.all_confs;
//     all_persons = s.all_persons;
//     all_maps    = s.all_maps;
//     all_texts   = s.all_texts;
// 
//     logged_in_person = s.logged_in_person;
//     session_number   = s.session_number;
// 
//     return *this;
// }


// ostream &
// operator <<(ostream &os, const Session &s)
// {
//     cout << "A session object";
//     return os;
// }


//void
//Session::set_connection(Connection *connection)
//{
//    conn = connection;
//    //+++ Reset confs, persons, maps and texts here.
//}


// ================================================================


Conferences & 
Session::conferences()
{
    assert(conn != NULL);

    if (all_confs == NULL) {
	all_confs = new Conferences(conn, m_async_dispatcher);
    }

    return *all_confs;
}


Persons &
Session::persons()
{
    if (all_persons == NULL) {
	all_persons = new Persons(conferences());
    }

    return *all_persons;
}


Maps &
Session::maps()
{
    if (all_maps == NULL) {
	all_maps = new Maps(conferences(), m_async_dispatcher);
    }

    return *all_maps;
}


Texts &  
Session::texts()
{
    assert(conn != NULL);

    if (all_texts == NULL) {
	all_texts = new Texts(conn, m_async_dispatcher);
    }

    return *all_texts;

}


// ----------------------------------------------------------------

Status
Session::create_person(const LyStr &name,
		       const LyStr &password,
		       Pers_no *result,
		       bool *autologin)
{
    create_person_question q(conn, name, password);
    
    Status ret = q.receive();
    if (ret == st_ok)
    {
	if (result != NULL)
	    *result = q.result();
	// If you are not already logged in, an automatic login occurs.
	if (logged_in_person == 0)
	{
	    logged_in_person = q.result();
	    if (autologin != NULL)
		*autologin = true;
	}
	else
	    if (autologin != NULL)
		*autologin = false;
    }

    return ret;
}

Status
Session::login(Pers_no person, const LyStr &passwd, bool invisible)
{
    login_question in(conn, person, passwd, invisible);

    // +++ FIXME: Delete all caches!  Should this be a separate method?
    Status ret = in.receive();
    if (ret == st_ok)
	logged_in_person = person;
    else
	logged_in_person = 0;

    return ret;
}

Status
Session::add_member(Conf_no cno, 
		    Pers_no reader,
		    unsigned char priority, 
		    unsigned short placement)
{
    add_member_question q(conn, cno, reader, priority, placement);
    return q.receive();
}

Status
Session::send_message(Pers_no recipient, 
		      const LyStr &message)
{
    // FIXME: Loses error code.
    send_message_question q(conn, recipient, message);
    return q.receive();
}

struct tm
Session::get_time()
{
    get_time_question   timeq(conn);
    Status              stat = timeq.receive();

    if (stat == st_ok)
	return timeq.result();

    // +++ Error handling!!
    return timeq.result();
}


Status
Session::change_what_i_am_doing(const LyStr &str)
{
    change_what_i_am_doing_question  doing(conn, str);

    return doing.receive();
}


Array<Who_info>
Session::who_is_on()
{
    who_is_on_question   who_q(conn);
    
    if (who_q.receive() == st_ok)
	return who_q.result();
    else
	abort();		// +++Error handling!
}


Array<Micro_conf>
Session::lookup_name(const LyStr &pattern)
{
    lookup_name_question   lookupq(conn, pattern);

    if (lookupq.receive() == st_ok) 
	return lookupq.result();
    else
	abort();		// +++Error handling!
}

Array<Conf_z_info>
Session::lookup_z_name(const LyStr &pattern,
		       bool want_persons,
		       bool want_confs)
{
    lookup_z_name_question   lookupq(conn, pattern, want_persons, want_confs);

    if (lookupq.receive() == st_ok) 
	return lookupq.result();
    else
	abort();		// +++Error handling!
}


Array<Pers_no>
Session::regexp_lookup_persons(const LyStr &pattern)
{
    re_lookup_person_question   lookupq(conn, pattern);

    if (lookupq.receive() == st_ok) 
	return lookupq.result();
    else
	abort();		// +++Error handling!
}


Array<Mark>
Session::get_marks()
{
    get_marks_question  markq(conn);

    if (markq.receive() == st_ok) 
	return markq.result();
    else
	abort();		// +++Error handling!
}

Pers_no
Session::my_login() const
{
    return logged_in_person;
}

Session_no
Session::my_session() const
{
    if (session_number == 0)
    {
	who_am_i_question wq(conn);
	if (wq.receive() == st_ok)
	    ((Session*)this)->session_number = wq.result();
	else
	    abort();		// FIXME: Error handling
    }

    return session_number;
}


Async_dispatcher &
Session::async_dispatcher()
{
    return *m_async_dispatcher;
}


Status
Session::handle_async(int this_many)
{
    return m_async_dispatcher->handle_async(this_many);
}
